//                                                                                
//    Copyright (c) 2004/2005 Thomas Fuchs / The Rockin'-B, www.rockin-b.de       
//                                                                                
//    Permission is granted to anyone to use this software for any purpose        
//    and to redistribute it freely, subject to the following restrictions:       
//                                                                                
//    1.  The origin of this software must not be misrepresented. You must not    
//        claim that you wrote the original software. If you use this software    
//        in a product, an acknowledgment in the product documentation would be   
//        appreciated but is not required.                                        
//                                                                                
//    2.  ANY COMMERCIAL USE IS PROHIBITED. This includes that you must not use   
//        this software in the production of a commercial product. Only an explicit
//        permission by the author can allow you to do that.                      
//                                                                                
//    3.  Any modification applied to the software must be marked as such clearly.
//                                                                                
//    4.  This license may not be removed or altered from any distribution.       
//                                                                                
//                                                                                
//    			    NO WARRANTY                                                 
//                                                                                
//    THERE IS NO WARRANTY FOR THE PROGRAM.                                       
//    THE PROGRAM IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,               
//    EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,                 
//    THE IMPLIED WARRANTIES OF FITNESS FOR A PARTICULAR PURPOSE.                 
//    THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.
//    SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY    
//    SERVICING, REPAIR OR CORRECTION.                                            
//                                                                                
//    IN NO EVENT WILL THE COPYRIGHT HOLDER BE LIABLE TO YOU FOR DAMAGES,         
//    INCLUDING ANY GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING 
//    OUT OF THE USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED   
//    TO LOSS OF DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY    
//    YOU OR THIRD PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER  
//    PROGRAMS), EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE       
//    POSSIBILITY OF SUCH DAMAGES.                                                
//                                                                                

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "portingGNUBoy.h"
#include "lcd.h"

#include <SGL.H>
#include <RB_were_here.h>
#include <RB_CartRAM.h>
#include <RB_minCD.h>

extern Uint16 framebuffer[];

void initializeVDP2FB(unsigned int options)
{
// the address for background color in VRAM is taken from various examples,
// don't know why
// !!! beware, 1st try the bgcol was the only thing to see on real Saturn
// what exactly is the back_col_adr? maybe the color at this adress is 
// interpreted as background color and wheereever such color occurs,
// it's replaced by the given one
// !! all i see is the background color....
    slBack1ColSet((void *)BACK_CRAM, CD_Black);
    
    
// the debug text lib take the NBG0 scroll <- the only one with 24bits color depth
// priority = 0 -> scroll not displayed
// take farest away    
    slPriorityNbg0(7);
    slPriorityNbg1(6);
//    slPrioritySpr0-7(0);    

// we want bitmap scrolls
// only NBG0 and NBG1 can handle bitmap mode
// color modes:
//      palettet:           COL_TYPE_16, COL_TYPE_256, COL_TYPE_2048
//      rgb:                COL_TYPE_32768, COL_TYPE_1M
// bitmap sizes(BM_HxV):    BM_512x256, BM_512x512, BM_1024x256, BM_1024x512
    slBitMapNbg1(COL_TYPE_32768, BM_512x256, (void *)VDP2_VRAM_A0);
    memset((void *)VDP2_VRAM_A0, CD_Black, 256*512*2);

// monitor position in scroll coordinate space
// sets the position of monitors topleft corner(fixed format)
    slScrPosNbg1(toFIXED(-16.0), toFIXED(0.0));

// reduction setting: only NGB0 and NBG1 are able to RBG, too)
// modes:   ZOOM_1, ZOOM_HALF, ZOOM_QUATER      each untill x256
    slZoomModeNbg1(ZOOM_1);
    
// zoom
// fixed point reziprocal of scale ratio in x and y direction
    slZoomNbg1(toFIXED(0.6), toFIXED(0.6));            

// registeres scrolls
// call only after all scroll function settings
// returns 0 or -1(ERROR)
    slScrAutoDisp(NBG0ON | NBG1ON);
}

void initializeDisplay(unsigned int options)
{
// caution: when setting to 2, strange errors occur in satourne
    slInitSystem(TV_320x240, (TEXTURE *)NULL, 1);
//    slInitSystem(TV_320x224, NULL, 2);
    
    slTVOff();

    initializeVDP2FB(options);
    
    slTVOn();
}


void ss_main(void)
{
    Uint16 data;
    Sint16 CartID;
    // configure MINCD_load
    // decreasing priority
    FIO_ftype ftypes[] = {
        {"*.gb", {0, 0x400000}, "GameBoy rom", FIO_OP_LOAD},
        {"*.gbc", {0, 0x400000}, "GameBoy color rom", FIO_OP_LOAD},
        {"*.cgb", {0, 0x400000}, "GameBoy color rom", FIO_OP_LOAD},
    };
    FIO_in in = {ftypes, sizeof(ftypes)/sizeof(FIO_ftype), NULL, 0};
    FIO_out out;
    int i;
    
    initializeDisplay(0);
    
    EMST_init(4, 256, 3);
    EMST_name(EMST_CPU, "cpu");
    EMST_name(EMST_LCD, "lcd");
    EMST_name(EMST_VRAM, "vram");
    EMST_name(EMST_PAD, "pad");

    // setup CartRAM
#ifndef PORTING_ROM_INCLUDED    
        // 8MB RAM has only 4MB consecutive data
    CartID = RB_CartRAM_init(0, 0);
    if(CartID == CARTRAM_ID_32M) {
        // rom is loaded to cart ram
        in.dest = CartRAM0;
        in.destSize = CartRAMsize << 1;    // both banks form consecutive memory
    } else {
        // rom goes into low workRAM
        in.dest = (void *)0x200000;
        in.destSize = 0x100000;
        slPrint("Without 4MB RAM cart,", slLocate(4, 20));
        slPrint("only 1MByte roms possible!", slLocate(4, 21));
        slPrint("PRESS START BUTTON", slLocate(8, 23));
        while((Smpc_Peripheral[0].data & PER_DGT_ST) != 0)
            slSynch();
    }
#endif

    MINCD_init();

    while(1) {
#ifndef PORTING_ROM_INCLUDED
        // load rom into low workRAM / Cart RAM respectively
        clearText();
        do{
            MINCD_load("Select a ROM:", &in, &out);
        } while (out.value.size == 0);    
        // if it fits into low workRAM
        // take it there, it's faster
        gamedata_size = out.value.size;
        if((CartID == CARTRAM_ID_32M) && (gamedata_size <= 0x100000)) {
            memcpy((void *)0x200000, in.dest, gamedata_size);
            gamedata = (void *)0x200000;
        } else {
            gamedata = in.dest;
        }
#endif        
        clearText();
        slPrint("Rockin'-B's GNUBoy", slLocate(8, 3));
        slPrint("frameskip", slLocate(2, 27));
        slPrint("frame", slLocate(2, 28));

        // setup GNUBoy
        gb_vid_init ();
        pcm_init ();
        mem_init ();

        // loading rom: run_game function
        if (loader_init("no file..")) {
            emu_reset();
//        were_here("after reset, pc: %x", PC);
            emu_run();
        
            pcm_clear_buffer();
            loader_unload();
        }
        gb_vid_close();
        pcm_close();
        EMST_reset();
        // screen clear
        memset((void *)VDP2_VRAM_A0, CD_Black, 256*512*2);        
        memset(framebuffer, 0, 144*LCD_BUF_WIDTH*sizeof(Uint16));        
    }    

    while(1) {
        // text display
        slPrint("Rockin'-B's GNUBoy: exited", slLocate(5, 4));

        slSynch();
    }    
}


#if 0
#include <SGL.H>
/*********************************/
/* taken from cinit.c */
/*********************************/
#define		SystemWork		0x060ffc00		/* System Variable Address */
#define		SystemSize		(0x06100000-0x060ffc00)		/* System Variable Size */
/* sl.lnk Ŏw肵.bssZNV̊JnAIV{ */
extern Uint32 _bstart, _bend;
 
int main( void )
{
	Uint8	*dst;
	Uint32	i; 

	/* 1.Zero Set .bss Section */
	for( dst = (Uint8 *)&_bstart; dst < (Uint8 *)&_bend; dst++ ) {
		*dst = 0;
	}
	/* 2.ROM has data at end of text; copy it. */

	/* 3.SGL System Variable Clear */
	for( dst = (Uint8 *)SystemWork, i = 0;i < SystemSize; i++) {
		*dst = 0;
	}

	/* Application Call */
	ss_main();
	return 1;
}
#endif
